package fr.imag.adele.ant.templatetask;

import java.util.NoSuchElementException;

/**
 * The string tokenizer class allows an application to break a 
 * string into tokens. The tokenization method differes from those
 * used by the <code>java.util.StringTokenizer</code> class. The 
 * <code>StringTokenizer</code> method considers tokens as string seperated by the delimiter string.
 * A sequence of two delimeters is considered as a empty token. 
 * <p>
 * The following is one example of the use of the tokenizer. The code:
 * <blockquote><pre>
 *     StringTokenizer st = new StringTokenizer("john;smith;js@dummy.org",";");
 *     while (st.hasMoreTokens()) {
 *         println(st.nextToken());
 *     }
 * </pre></blockquote>
 *
 * @author  Didier Donsez
 * @see     java.util.StringTokenizer
 */
public class StringTokenizer {
	private int currentPosition;
	private int maxPosition;
	private String str;
	private String delimiter;
	private int delimiterLen;

	/**
	 * Constructs a string tokenizer for the specified string. The 
	 * characters in the <code>delim</code> argument are the delimiter 
	 * for separating tokens. 
	 * <p>
	 *
	 * @param   str            a string to be parsed.
	 * @param   delimiter      the delimiter string that separate tokens.
	 */
	public StringTokenizer(String str, String delimiter) {
		currentPosition = 0;
		maxPosition = str.length();
		this.str = str;
		this.delimiter = delimiter;
		delimiterLen = delimiter.length();
	}

	/**
	 * Tests if there are more tokens available from this tokenizer's string.
	 *
	 * @return  <code>true</code> if there are more tokens available from this
	 *          tokenizer's string; <code>false</code> otherwise.
	 */
	public boolean hasMoreTokens() {
		return (currentPosition < maxPosition);
	}

	/**
	 * Returns the next token from this string tokenizer.
	 *
	 * @return     the next token from this string tokenizer.
	 * @exception  NoSuchElementException  if there are no more tokens in this
	 *               tokenizer's string.
	 */
	public String nextToken() {

		if (currentPosition >= maxPosition) {
			throw new NoSuchElementException();
		}

		int nextDelimiterPos=str.indexOf(delimiter, currentPosition);
		if(nextDelimiterPos>=0){
			// has more delimiter
			String token=str.substring(currentPosition,nextDelimiterPos);
			// skip delimiter
			currentPosition=nextDelimiterPos+delimiterLen;
			return token;
		} else {
			// last token
			String token=str.substring(currentPosition,maxPosition);
			currentPosition=maxPosition;
			return token;
		}
	}
}
