/*
 * Copyright  2004 Didier Donsez
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

package fr.imag.adele.ant.csvtoxml;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

public class CsvToXmlTask extends Task {

	private static final String ROW = "row";
	private static final String ID = "id";
		
	private File csvfile;
	private File tofile;
	private int firstid = 0;
	private int idincrement = 1;
	private String rowelement = ROW;
	private String idattribute = null;	
	private String rootelement = null;

	private static String CSV_DELIMITER = ";";

	public void execute() throws BuildException {
		if (csvfile == null) {
			throw new BuildException("csvfile must be specified");
		}

		if (tofile == null) {
			throw new BuildException("tofile must be specified");
		}

		BufferedReader csvbr;
		try {
			csvbr = new BufferedReader(new FileReader(csvfile));
		} catch (FileNotFoundException e) {
			throw new BuildException("File " + csvfile + " not found");
		}

		Writer writer;
		try {
			writer = new BufferedWriter(new FileWriter(tofile));
		} catch (IOException e1) {
			try {
				csvbr.close();
			} catch (IOException ignore) {
			}
			throw new BuildException(
				"Can't open file:" + tofile.getAbsolutePath());
		}

		StringBuffer sb = new StringBuffer();
		try {
			if(rootelement!=null) sb.append("<").append(rowelement).append(">\n");
			int currentid = firstid;

			String line = null;
			String headerline = null;

			while ((line = csvbr.readLine()) != null) {

				if (headerline == null) {
					headerline = line;
				} else {
					sb.append("<").append(rowelement);
					if(idattribute!=null) sb.append(" ").append(idattribute).append("=\"").append(currentid).append("\"");
					sb.append(">\n");
					currentid += idincrement;

					StringTokenizer headerst =
						new StringTokenizer(headerline, CSV_DELIMITER);
					StringTokenizer linest =
						new StringTokenizer(line, CSV_DELIMITER);

					while (headerst.hasMoreTokens()) {
						String headerfield = headerst.nextToken();
						if (linest.hasMoreTokens()) {
							String linefield = linest.nextToken();
							sb.append('<').append(headerfield).append('>');
							sb.append(linefield);
							sb.append("</").append(headerfield).append(">\n");
						} else {
							sb.append('<').append(headerfield).append("/>\n");
						}
					}
					sb.append("</").append(rowelement).append(">\n");
				}
			}
			if(rootelement!=null) sb.append("</").append(rowelement).append(">\n");
		} catch (IOException e) {
			throw new BuildException(
				"IO Error occured during reading file"
					+ csvfile.getAbsolutePath());
		} finally {
			if (csvbr != null) {
				try {
					csvbr.close();
				} catch (IOException ignore) {
				}
			}
		}

		try {
			writer.write(sb.toString());
			writer.close();
		} catch (IOException e2) {
			throw new BuildException(
				"Can't write into file:" + tofile.getAbsolutePath());
		} finally {
			if (writer != null) {
				try {
					writer.close();
				} catch (IOException ignore) {
				}
			}
		}
	}

	/**
	 * @param tofile the name of the generated file
	 */
	public void setTofile(File tofile) {
		this.tofile = tofile;
	}

	/**
	 * @param csvfile the data file in CSV (Comma-Separated Value) format
	 */
	public void setCsvfile(File csvfile) {
		this.csvfile = csvfile;
	}

	/**
	 * @param i	the first value of the id attribute
	 */
	public void setFirstid(int i) {
		firstid = i;
	}

	/**
	 * @param i the increment value of the id attribute
	 */
	public void setIdincrement(int i) {
		idincrement = i;
	}

	/**
	 * @param rowelement the name of the row element (yes, row)
	 */
	public void setRowelement(String rowelement) {
		this.rowelement = rowelement;
	}

	/**
	 * @param rootelement the name of the root element (no)
	 */
	public void setRootelement(String rootelement) {
		this.rootelement = rootelement;
	}

	/**
	 * @param idattribute the name of the row id attribute (yes, id)
	 */
	public void setIdattribute(String idattribute) {
		this.idattribute = idattribute;
	}

}
